@@ -4,84 +4,91 @@ package Net::BitTorrent::PeerPacket;
 
 use strict;
 use warnings;
-use Carp;
+use Carp qw(croak);
 use base 'Exporter';
 
-our $VERSION = '1.0';
+our $VERSION = '1.1';
 
-# Build list of packet names, order is significant, as the index of the type 
+# Build list of packet names, order is significant, as the index of the type
 # represents it's code in BitTorrent
-my( @code_num_to_str, %code_str_to_num );
+my ( @code_num_to_str, %code_str_to_num );
 
-BEGIN {    @code_num_to_str = qw(
-        BT_CHOKE
-        BT_UNCHOKE
-        BT_INTERESTED
-        BT_UNINTERESTED
-        BT_HAVE
-        BT_BITFIELD
-        BT_REQUEST
-        BT_PIECE
-        BT_CANCEL
+BEGIN {
+    @code_num_to_str = qw(
+      BT_CHOKE
+      BT_UNCHOKE
+      BT_INTERESTED
+      BT_UNINTERESTED
+      BT_HAVE
+      BT_BITFIELD
+      BT_REQUEST
+      BT_PIECE
+      BT_CANCEL
     );
 
     # The array @code_num_to_str serves as a packet code to packet name
     # map.  Flip that into a hash that serves as a name to code map
-    %code_str_to_num = 
-        map { $code_num_to_str[$_] => $_ } 0 .. $#code_num_to_str;
+    %code_str_to_num =
+      map { $code_num_to_str[$_] => $_ } 0 .. $#code_num_to_str;
+
+    $code_str_to_num{'BT_HANDSHAKE'} = -1;
 }
 
 # Turn the hash of name-id pairs into a bunch of constants based on the names
 use constant \%code_str_to_num;
 
 # Allow for the export of our build and parse subroutines and bt codes
-our @EXPORT_OK = ('bt_build_packet', 'bt_parse_packet', keys %code_str_to_num);
+our @EXPORT_OK =
+  ( 'bt_build_packet', 'bt_parse_packet', keys %code_str_to_num );
 
 # create an :all tag for subroutines and constants and a :constants tag for
 # just the constants
 our %EXPORT_TAGS = (
-    'all'       => [ @EXPORT_OK ],
-    'constants' => [ @code_num_to_str ],
+    'all'       => [@EXPORT_OK],
+    'constants' => [@code_num_to_str],
 );
 
 # Map build and parse subroutines
-my $bt_base_code = 0;
-my (%bt_dispatch) = map( { $bt_base_code++ => $_ } 
-    {
-        build   => sub { return _build_packet( BT_CHOKE ) },
-        parse   => { bt_code => BT_CHOKE },
+my $bt_base_code = -1;
+my (%bt_dispatch) = map( { $bt_base_code++ => $_ }{
+        build   => \&_build_handshake_packet,
+          parse => sub { croak('unimplemented') },
     },
     {
-        build   => sub { return _build_packet( BT_UNCHOKE ) },
-        parse   => { bt_code => BT_UNCHOKE },
+        build => sub       { return _build_packet(BT_CHOKE) },
+        parse => { bt_code => BT_CHOKE },
     },
     {
-        build   => sub { return _build_packet( BT_INTERESTED ) },
-        parse   => { bt_code => BT_INTERESTED },
+        build => sub       { return _build_packet(BT_UNCHOKE) },
+        parse => { bt_code => BT_UNCHOKE },
     },
     {
-        build   => sub { return _build_packet( BT_UNINTERESTED ) },
-        parse   => { bt_code => BT_UNINTERESTED },
+        build => sub       { return _build_packet(BT_INTERESTED) },
+        parse => { bt_code => BT_INTERESTED },
     },
     {
-        build   => \&_build_have_packet,
-        parse   => \&_parse_have_packet,
+        build => sub       { return _build_packet(BT_UNINTERESTED) },
+        parse => { bt_code => BT_UNINTERESTED },
     },
     {
-        build   => \&_build_bitfield_packet,
-        parse   => \&_parse_bitfield_packet,
+        build => \&_build_have_packet,
+        parse => \&_parse_have_packet,
     },
     {
-        build   => \&_build_request_packet,
-        parse   => \&_parse_request_packet,
+        build => \&_build_bitfield_packet,
+        parse => \&_parse_bitfield_packet,
     },
     {
-        build   => \&_build_piece_packet,
-        parse   => \&_parse_piece_packet,
+        build => \&_build_request_packet,
+        parse => \&_parse_request_packet,
     },
     {
-        build   => \&_build_cancel_packet,
-        parse   => \&_parse_cancel_packet,
+        build => \&_build_piece_packet,
+        parse => \&_parse_piece_packet,
+    },
+    {
+        build => \&_build_cancel_packet,
+        parse => \&_parse_cancel_packet,
     },
 );
 
@@ -92,40 +99,45 @@ my (%bt_dispatch) = map( { $bt_base_code++ => $_ }
 sub bt_build_packet {
 
     # hashify arguments
-    @_%2 == 0
-        or croak("Even number of elements expected, but not received");
+    @_ % 2 == 0
+      or croak("Even number of elements expected, but not received");
 
     my %args = @_;
 
-    _hash_defines(\%args, 'bt_code');
- 
+    _hash_defines( \%args, 'bt_code' );
+
     my $bt_code = $args{bt_code};
-    
+
     # look-up build subroutine
     defined $bt_dispatch{$bt_code}
-        or croak("Invalid BT code ($bt_code) found");
-    
+      or croak("Invalid BT code ($bt_code) found");
+
     my $sub_ref = $bt_dispatch{$bt_code}{build};
-    
+
     # execute the subroutine
-    return $sub_ref->(\%args);
+    return $sub_ref->( \%args );
 }
 
 sub bt_parse_packet {
     my ($packet_ref) = @_;
-    
-    my ($bt_code) = unpack('x4C', ${$packet_ref});
+
+    # handle the handshake if it is passed in
+    if ( unpack( 'c', ${$packet_ref} ) eq 0x13 ) {
+        return _parse_handshake_packet($packet_ref);
+    }
+
+    my ($bt_code) = unpack( 'x4C', ${$packet_ref} );
 
     defined $bt_dispatch{$bt_code}
-        or croak("Invalid BT code ($bt_code) found");
+      or croak("Invalid BT code ($bt_code) found");
 
     my $parse_ref = $bt_dispatch{$bt_code}{parse};
-   
+
     # for easy packets, we just get the hash ref back
-    return $parse_ref if(ref $parse_ref eq 'HASH');
-    
+    return $parse_ref if ( ref $parse_ref eq 'HASH' );
+
     # execute the subroutine
-    return $parse_ref->(\substr(${$packet_ref}, 5));
+    return $parse_ref->( \substr( ${$packet_ref}, 5 ) );
 }
 
 1;
@@ -134,14 +146,30 @@ sub bt_parse_packet {
 # P R I V A T E    S U B R O U T I N E S
 ##########################################################################
 
+#
+# _build_handshake_packet INFO_HASH PEER_ID
+#     Return a handshake packet
+#
+sub _build_handshake_packet {
+    my ($args) = @_;
+
+    _hash_defines( $args, 'info_hash', 'peer_id' );
+
+    my $packet = pack( 'c/a* a8 a20 a20',
+        'BitTorrent protocol',
+        '', $args->{info_hash}, $args->{peer_id} );
+
+    return $packet;
+}
+
 #
 # _build_have_packet PIECE_INDEX
 #     Return a have packet
 #
 sub _build_have_packet {
-    my($args) = @_;
+    my ($args) = @_;
 
-    _hash_defines($args, 'piece_index');
+    _hash_defines( $args, 'piece_index' );
 
     my $packet_body = pack( 'N', $args->{piece_index} );
 
@@ -153,9 +181,9 @@ sub _build_have_packet {
 #     Return a bitfield packet
 #
 sub _build_bitfield_packet {
-    my($args) = @_;
+    my ($args) = @_;
 
-    _hash_defines($args, 'bitfield_ref');
+    _hash_defines( $args, 'bitfield_ref' );
 
     return _build_packet( BT_BITFIELD, '', $args->{bitfield_ref} );
 }
@@ -165,15 +193,12 @@ sub _build_bitfield_packet {
 #     Return a request packet
 #
 sub _build_request_packet {
-    my($args) = @_;
+    my ($args) = @_;
 
-    _hash_defines($args, 'piece_index', 'block_offset', 'block_size');
+    _hash_defines( $args, 'piece_index', 'block_offset', 'block_size' );
 
-    my $packet_body = pack( 'NNN', 
-                            $args->{piece_index}, 
-                            $args->{block_offset}, 
-                            $args->{block_size} 
-                           );
+    my $packet_body = pack( 'NNN',
+        $args->{piece_index}, $args->{block_offset}, $args->{block_size} );
 
     return _build_packet( BT_REQUEST, $packet_body );
 }
@@ -183,14 +208,11 @@ sub _build_request_packet {
 #     Return a piece packet
 #
 sub _build_piece_packet {
-    my($args) = @_;
+    my ($args) = @_;
 
-    _hash_defines($args, 'piece_index', 'block_offset', 'data_ref');
+    _hash_defines( $args, 'piece_index', 'block_offset', 'data_ref' );
 
-    my $packet_body = pack( 'NN', 
-                            $args->{piece_index}, 
-                            $args->{block_offset}
-                           );
+    my $packet_body = pack( 'NN', $args->{piece_index}, $args->{block_offset} );
 
     return _build_packet( BT_PIECE, $packet_body, $args->{data_ref} );
 }
@@ -200,15 +222,12 @@ sub _build_piece_packet {
 #     Return a cancel packet
 #
 sub _build_cancel_packet {
-    my($args) = @_;
+    my ($args) = @_;
 
-    _hash_defines($args, 'piece_index', 'block_offset', 'block_size');
+    _hash_defines( $args, 'piece_index', 'block_offset', 'block_size' );
 
-    my $packet_body = pack( 'NNN', 
-                            $args->{piece_index}, 
-                            $args->{block_offset}, 
-                            $args->{block_size} 
-                           );
+    my $packet_body = pack( 'NNN',
+        $args->{piece_index}, $args->{block_offset}, $args->{block_size} );
     return _build_packet( BT_CANCEL, $packet_body );
 }
 
@@ -221,32 +240,47 @@ sub _build_cancel_packet {
 sub _build_packet {
     my ( $_code, $packet_body, $data_ref ) = @_;
 
-    $packet_body = ''   unless defined $packet_body;
-    $data_ref    = \ '' unless defined $data_ref;
+    $packet_body = ''  unless defined $packet_body;
+    $data_ref    = \'' unless defined $data_ref;
 
-    my $packet = pack(
-        'NCa*a*',
-        length( $packet_body ) + length(${$data_ref}) + 1,
-        $_code,
-        $packet_body,
-        ${$data_ref}    );
+    my $packet = pack( 'NCa*a*',
+        length($packet_body) + length( ${$data_ref} ) + 1,
+        $_code, $packet_body, ${$data_ref} );
 
     return $packet;
 }
 
+#
+# _parse_handshake_packet ENTIRE_PACKET
+#     Return a parsed handshake packet
+#
+sub _parse_handshake_packet {
+    my ($packet_ref) = @_;
+
+    my ( $protocol_name, $reserved_space, $info_hash, $peer_id ) =
+      unpack( 'c/a* a8 a20 a20', ${$packet_ref} );
+
+    return {
+        bt_code   => BT_HANDSHAKE,
+        protocol  => $protocol_name,
+        info_hash => $info_hash,
+        peer_id   => $peer_id,
+    };
+}
+
 #
 # _parse_have_packet PACKET_PAYLOAD
 #     Return a parsed have packet
 #
 sub _parse_have_packet {
     my ($packet_ref) = @_;
-    
-    my ($piece_index) = unpack('N', $$packet_ref);
-        
-    return { 
-             bt_code     => BT_HAVE, 
-             piece_index => $piece_index 
-           };
+
+    my ($piece_index) = unpack( 'N', $$packet_ref );
+
+    return {
+        bt_code     => BT_HAVE,
+        piece_index => $piece_index
+    };
 }
 
 #
@@ -255,11 +289,11 @@ sub _parse_have_packet {
 #
 sub _parse_bitfield_packet {
     my ($packet_ref) = @_;
-    
-    return { 
-             bt_code      => BT_BITFIELD, 
-             bitfield_ref => $packet_ref,
-           };
+
+    return {
+        bt_code      => BT_BITFIELD,
+        bitfield_ref => $packet_ref,
+    };
 }
 
 #
@@ -268,16 +302,16 @@ sub _parse_bitfield_packet {
 #
 sub _parse_request_packet {
     my ($packet_ref) = @_;
-    
-    my ($piece_index, $block_offset, $block_size) 
-        = unpack('NNN', $$packet_ref);
-        
-    return { 
-             bt_code      => BT_REQUEST, 
-             piece_index  => $piece_index,
-             block_offset => $block_offset,
-             block_size   => $block_size,
-           };
+
+    my ( $piece_index, $block_offset, $block_size ) =
+      unpack( 'NNN', $$packet_ref );
+
+    return {
+        bt_code      => BT_REQUEST,
+        piece_index  => $piece_index,
+        block_offset => $block_offset,
+        block_size   => $block_size,
+    };
 }
 
 #
@@ -286,15 +320,15 @@ sub _parse_request_packet {
 #
 sub _parse_piece_packet {
     my ($packet_ref) = @_;
-    
-    my ($piece_index, $block_offset) = unpack('NN', $$packet_ref);
-    
-    return { 
-             bt_code      => BT_PIECE, 
-             piece_index  => $piece_index,
-             block_offset => $block_offset,
-             data_ref     => \substr($$packet_ref, 8),
-           };
+
+    my ( $piece_index, $block_offset ) = unpack( 'NN', $$packet_ref );
+
+    return {
+        bt_code      => BT_PIECE,
+        piece_index  => $piece_index,
+        block_offset => $block_offset,
+        data_ref     => \substr( $$packet_ref, 8 ),
+    };
 }
 
 #
@@ -303,15 +337,16 @@ sub _parse_piece_packet {
 #
 sub _parse_cancel_packet {
     my ($packet_ref) = @_;
-    
-    my ($piece_index, $block_offset, $block_size) = unpack('NNN', $$packet_ref);
-        
-    return { 
-             bt_code      => BT_CANCEL, 
-             piece_index  => $piece_index,
-             block_offset => $block_offset,
-             block_size   => $block_size,
-           };
+
+    my ( $piece_index, $block_offset, $block_size ) =
+      unpack( 'NNN', $$packet_ref );
+
+    return {
+        bt_code      => BT_CANCEL,
+        piece_index  => $piece_index,
+        block_offset => $block_offset,
+        block_size   => $block_size,
+    };
 }
 
 #
@@ -319,16 +354,16 @@ sub _parse_cancel_packet {
 #     Makes sure that the given hash defines values for all keys in the list.
 #
 sub _hash_defines {
-    my ($hash, @keys) = @_;
-    
+    my ( $hash, @keys ) = @_;
+
     ref $hash eq 'HASH'
-        or croak("Hash reference not found");
-        
+      or croak("Hash reference not found");
+
     for my $key (@keys) {
         defined $hash->{$key}
-            or croak("$key not specified");
+          or croak("$key not specified");
     }
-    
+
     return 1;
 }
 
@@ -342,6 +377,12 @@ sub _hash_defines {
 
 =head1 SYNOPSIS 
 
+  # import everything
+  use Net::BitTorrent::PeerPacket qw(:all);
+
+  # or be more selective
+  use Net::BitTorrent::PeerPacket qw(bt_build_packet :constants);
+
   # Encode a packet
   my $binary_packet = bt_build_packet($key1, $value1, $key2, $value2);
 
@@ -351,36 +392,60 @@ sub _hash_defines {
 =head1 DESCRIPTION
 
 C<Net::BitTorrent::PeerPacket> handles parsing and building binary data 
-shared between BitTorrent peers.  The module optinally exports a single
+shared between BitTorrent peers.  The module optionally exports a single
 subroutine for building packets and another for parsing packets, as well
 as, a constant for each packet type defined by BitTorrent.
 
 =head1 CONSTANTS
 
-There are nine primary types of packets that are shared between peers on a
+There are ten primary types of packets that are shared between peers on a
 BitTorrent network.  The following constants are how the type of packet
 being build/parsed are represented within this module.
 
 =over 4
 
+=item BT_HANDSHAKE
+
+Used to start communication between peers.
+
 =item BT_CHOKE
 
+Tell a peer that it is choked.
+
 =item BT_UNCHOKE
 
+Tell a peer that it is unchoked.
+
 =item BT_INTERESTED
 
+Used to tell a peer that it has a piece that you need.
+
 =item BT_UNINTERESTED
 
+Used to tell a peer that it has no pieces that you need.
+
 =item BT_HAVE
 
+Used to tell a peer that you now have a specific piece.
+
 =item BT_BITFIELD
 
+Used right after a handshake, this tells a peer all of the pieces
+that you have and don't have in one message.
+
 =item BT_REQUEST
 
+Used to request a block of data from a piece that a peer has.
+
 =item BT_PIECE
 
+Used to return a block of data that was requested.
+
 =item BT_CANCEL
 
+Used to tell a peer that you no longer need the piece that you
+were downloading from them.
+
 =back
 
 =head1 SUBROUTINES
@@ -392,39 +457,58 @@ The arguments are passed into the subroutine as a list of key-value pairs.
 The resultant packet is sent back as a scalar.
 
 Depending on the requested packet type, the required arguments vary.  One 
-argument that is common to all calls is the bt_code.  The bt_code maps to a 
-BT_ constant exported by this module and determines the type of packet that
-will be built.
+argument that is common to all calls is the C<bt_code>.  The C<bt_code> maps 
+to a C<BT_> constant exported by this module and determines the type of 
+packet that will be built.
 
 What follows is a list of the different BT codes and the details of calling
 this subroutine with those codes.
 
+=head3 BT_HANDSHAKE
+
+Passing the C<BT_HANDSHAKE> code causes a handshake packet to be generated.  
+This type of packet is sent as soon as peers are connected and requires two
+additional keys:
+
+=over 4
+
+=item * info_hash
+
+The hash found in the C<.torrent> file that represents the download.
+
+=item * peer_id
+
+The peer ID for the local peer.  This should be the same as what is reported
+to the tracker for the swarm.
+
+=back
+
 =head3 BT_CHOKE
 
-Passing the BT_CHOKE code causes a choke packet to be generated.  This type of
+Passing the C<BT_CHOKE> code causes a choke packet to be generated.  This type of
 packet requires no additional data and therefore no additional arguments.
 
 =head3 BT_UNCHOKE
 
-Passing the BT_UNCHOKE code causes an unchoke packet to be generated.  This 
+Passing the C<BT_UNCHOKE> code causes an unchoke packet to be generated.  This 
 type of packet requires no additional data and therefore no additional 
 arguments.
 
 =head3 BT_INTERESTED
 
-Passing the BT_INTERESTED code causes an interested packet to be generated.  
+Passing the C<BT_INTERESTED> code causes an interested packet to be generated.  
 This type of packet requires no additional data and therefore no additional 
 arguments.
 
 =head3 BT_UNINTERESTED
 
-Passing the BT_UNINTERESTED code causes an uninterested packet to be generated.  
+Passing the C<BT_UNINTERESTED> code causes an uninterested packet to be generated.  
 This type of packet requires no additional data and therefore no additional 
 arguments.
 
 =head3 BT_HAVE
 
-Passing the BT_HAVE code causes a have packet to be generated.  This type of 
+Passing the C<BT_HAVE> code causes a have packet to be generated.  This type of 
 packet requires a piece index in addition to the BT code.
 
 =over 4
@@ -437,7 +521,7 @@ The piece index is the zero-based numeric index of a piece within a torrent.
 
 =head3 BT_BITFIELD
 
-Passing the BT_BITFIELD code causes a bit field packet to be generated.  This 
+Passing the C<BT_BITFIELD> code causes a bit field packet to be generated.  This 
 type of packet requires the bit field be specified in addition to the BT code.
 
 =over 4
@@ -451,7 +535,7 @@ binary data representing the pieces that are present and missing.
 
 =head3 BT_REQUEST
 
-Passing the BT_REQUEST code causes a request packet to be generated.  This 
+Passing the C<BT_REQUEST> code causes a request packet to be generated.  This 
 type of packet requires the piece index along with block offset and size in 
 addition to the BT code.
 
@@ -476,9 +560,9 @@ too big.
 
 =head3 BT_PIECE
 
-Passing the BT_PIECE code causes a piece packet to be generated.  This 
+Passing the C<BT_PIECE> code causes a piece packet to be generated.  This 
 type of packet requires the piece index along with block offset and the data
-to be transfered in addition to the BT code.
+to be transferred in addition to the BT code.
 
 =over 4
 
@@ -500,7 +584,7 @@ specified block offset within the specified piece.
 
 =head3 BT_CANCEL
 
-Passing the BT_CANCEL code causes a cancel packet to be generated.  This 
+Passing the C<BT_CANCEL> code causes a cancel packet to be generated.  This 
 type of packet requires the piece index along with block offset and size in 
 addition to the BT code.
 
@@ -539,25 +623,25 @@ additional keys that will be packaged with each code.
 
 =head3 BT_CHOKE
 
-The resultant hash from a choke packet will only contain the 'bt_code' key.
+The resultant hash from a choke packet will only contain the C<bt_code> key.
 
 =head3 BT_UNCHOKE
 
-The resultant hash from an unchoke packet will only contain the 'bt_code' key.
+The resultant hash from an unchoke packet will only contain the C<bt_code> key.
 
 =head3 BT_INTERESTED
 
-The resultant hash from an interested packet will only contain the 'bt_code' 
+The resultant hash from an interested packet will only contain the C<bt_code> 
 key.
 
 =head3 BT_UNINTERESTED
 
-The resultant hash from an uninterested packet will only contain the 'bt_code' 
+The resultant hash from an uninterested packet will only contain the C<bt_code> 
 key.
 
 =head3 BT_HAVE
 
-The resultant hash from a have packet will only contain the 'bt_code' 
+The resultant hash from a have packet will only contain the C<bt_code> 
 key and the following additional keys.
 
 =over 4
@@ -570,7 +654,7 @@ The piece index is the zero-based numeric index of a piece within a torrent.
 
 =head3 BT_BITFIELD
 
-The resultant hash from a bit field packet will only contain the 'bt_code' 
+The resultant hash from a bit field packet will only contain the C<bt_code> 
 key and the following additional keys.
 
 =over 4
@@ -584,7 +668,7 @@ binary data representing the pieces that are present and missing.
 
 =head3 BT_REQUEST
 
-The resultant hash from a request packet will only contain the 'bt_code' 
+The resultant hash from a request packet will only contain the C<bt_code> 
 key and the following additional keys.
 
 =over 4
@@ -608,7 +692,7 @@ too big.
 
 =head3 BT_PIECE
 
-The resultant hash from a piece packet will only contain the 'bt_code' 
+The resultant hash from a piece packet will only contain the C<bt_code> 
 key and the following additional keys.
 
 =over 4
@@ -631,7 +715,7 @@ specified block offset within the specified piece.
 
 =head3 BT_CANCEL
 
-The resultant hash from a cancel packet will only contain the 'bt_code' 
+The resultant hash from a cancel packet will only contain the C<bt_code> 
 key and the following additional keys.
 
 =over 4
